@@ -1,3 +1,6 @@
+"""
+N-Beats model for timeseries forecasting without covariates.
+"""
 from typing import Dict, List, Any
 import numpy as np
 import matplotlib.pyplot as plt
@@ -15,12 +18,12 @@ from pytorch_forecasting.models.nbeats.sub_modules import NBEATSTrendBlock, NBEA
 class NBeats(BaseModel):
     def __init__(
         self,
-        stack_types: List[str] = ["T", "S", "G"],
-        num_blocks=[3, 3, 1],
-        num_block_layers=[3, 3, 3],
+        stack_types: List[str] = ["trend", "seasonality"],
+        num_blocks=[3, 3],
+        num_block_layers=[3, 3],
         widths=[16, 128, 16],
-        sharing: List[int] = [True, True, False],
-        expansion_coefficient_lengths: List[int] = [5, 7, 4],
+        sharing: List[int] = [True, True],
+        expansion_coefficient_lengths: List[int] = [5, 7],
         prediction_length: int = 1,
         context_length: int = 1,
         dropout: float = 0.1,
@@ -34,12 +37,12 @@ class NBeats(BaseModel):
         **kwargs,
     ):
         """
-        Initialize NBeats Model
+        Initialize NBeats Model - use its :py:meth:`~from_dataset` method if possible.
 
         Args:
-            stack_types: One of the following values: “G” (generic), “S” (seasonal) or “T” (trend). A list of strings
+            stack_types: One of the following values: “generic”, “seasonality" or “trend". A list of strings
                 of length 1 or ‘num_stacks’. Default and recommended value
-                for generic mode: [“G”] Recommended value for interpretable mode: [“T”,”S”]
+                for generic mode: [“generic”] Recommended value for interpretable mode: [“trend”,”seasonality”]
             num_blocks: The number of blocks per stack. A list of ints of length 1 or ‘num_stacks’.
                 Default and recommended value for generic mode: [1] Recommended value for interpretable mode: [3]
             num_block_layers: Number of fully connected layers with ReLu activation per block. A list of ints of length
@@ -72,7 +75,7 @@ class NBeats(BaseModel):
         self.net_blocks = nn.ModuleList()
         for stack_id, stack_type in enumerate(stack_types):
             for _ in range(num_blocks[stack_id]):
-                if stack_type == "G":
+                if stack_type == "generic":
                     net_block = NBEATSGenericBlock(
                         units=self.hparams.widths[stack_id],
                         thetas_dim=self.hparams.expansion_coefficient_lengths[stack_id],
@@ -81,7 +84,7 @@ class NBeats(BaseModel):
                         forecast_length=prediction_length,
                         dropout=self.hparams.dropout,
                     )
-                elif stack_type == "S":
+                elif stack_type == "seasonality":
                     net_block = NBEATSSeasonalBlock(
                         units=self.hparams.widths[stack_id],
                         num_block_layers=self.hparams.num_block_layers[stack_id],
@@ -90,7 +93,7 @@ class NBeats(BaseModel):
                         min_period=self.hparams.expansion_coefficient_lengths[stack_id],
                         dropout=self.hparams.dropout,
                     )
-                elif stack_type == "T":
+                elif stack_type == "trend":
                     net_block = NBEATSTrendBlock(
                         units=self.hparams.widths[stack_id],
                         thetas_dim=self.hparams.expansion_coefficient_lengths[stack_id],
@@ -104,13 +107,25 @@ class NBeats(BaseModel):
 
                 self.net_blocks.append(net_block)
 
-    def forward(self, x: Dict[str, torch.Tensor]):
-        target = x["encoder_target"]
+    def forward(self, x: Dict[str, torch.Tensor]) -> Dict[str, torch.Tensor]:
+        """
+        Pass forward of network.
+
+        Args:
+            x (Dict[str, torch.Tensor]): input from dataloader generated from :py:class:`~TimeSeriesDataSet`.
+
+        Returns:
+            Dict[str, torch.Tensor]: output of model
+        """
+        target = x["encoder_cont"][..., 0]
 
         timesteps = self.hparams.context_length + self.hparams.prediction_length
         generic_forecast = [torch.zeros((target.size(0), timesteps), dtype=torch.float32, device=self.device)]
         trend_forecast = [torch.zeros((target.size(0), timesteps), dtype=torch.float32, device=self.device)]
         seasonal_forecast = [torch.zeros((target.size(0), timesteps), dtype=torch.float32, device=self.device)]
+        forecast = torch.zeros(
+            (target.size(0), self.hparams.prediction_length), dtype=torch.float32, device=self.device
+        )
 
         backcast = target  # initialize backcast
         for i, block in enumerate(self.net_blocks):
@@ -130,10 +145,7 @@ class NBeats(BaseModel):
             backcast = (
                 backcast - backcast_block
             )  # do not use backcast -= backcast_block as this signifies an inline operation
-            if i == 0:
-                forecast = forecast_block
-            else:
-                forecast = forecast + forecast_block
+            forecast = forecast + forecast_block
 
         return dict(
             prediction=forecast,
@@ -146,6 +158,16 @@ class NBeats(BaseModel):
 
     @classmethod
     def from_dataset(cls, dataset: TimeSeriesDataSet, **kwargs):
+        """
+        Convenience function to create network from :py:class`~TimeSeriesDataSet`.
+
+        Args:
+            dataset (TimeSeriesDataSet): dataset where sole predictor is the target.
+            **kwargs: additional arguments to be passed to ``__init__`` method.
+
+        Returns:
+            NBeats
+        """
         new_kwargs = {"prediction_length": dataset.max_prediction_length, "context_length": dataset.max_encoder_length}
         new_kwargs.update(kwargs)
 
@@ -161,19 +183,30 @@ class NBeats(BaseModel):
         assert dataset.randomize_length is None, "length has to be fixed, but randomize_length is not None"
         assert not dataset.add_relative_time_idx, "add_relative_time_idx has to be False"
 
+        assert (
+            len(dataset.flat_categoricals) == 0
+            and len(dataset.reals) == 1
+            and len(dataset.time_varying_unknown_reals) == 1
+            and dataset.time_varying_unknown_reals[0] == dataset.target
+        ), "The only variable as input should be the target which is part of time_varying_unknown_reals"
+
         # initialize class
         return super().from_dataset(dataset, **new_kwargs)
 
     def step(self, x, y, batch_idx, label) -> Dict[str, torch.Tensor]:
+        """
+        Take training / validation step.
+        """
         log, out = super().step(x, y, batch_idx=batch_idx, label=label)
         self._log_interpretation(x, out, batch_idx=batch_idx, label=label)
         return log, out
 
     def _log_interpretation(self, x, out, batch_idx, label="train"):
+        """
+        Log interpretation of network predictions in tensorboard.
+        """
         if self.log_interval(label == "train") > 0 and batch_idx % self.log_interval(label == "train") == 0:
-            fig = self.plot_interpretation(
-                {name: value[0] for name, value in x.items()}, {name: value[0] for name, value in out.items()},
-            )
+            fig = self.plot_interpretation(x, out, idx=0)
             name = f"{label.capitalize()} interpretation of item 0 in "
             if label == "train":
                 name += f"step {self.global_step}"
@@ -181,19 +214,50 @@ class NBeats(BaseModel):
                 name += f"batch {batch_idx}"
             self.logger.experiment.add_figure(name, fig, global_step=self.global_step)
 
-    def plot_interpretation(self, x, output):
-        fig, ax = plt.subplots(2, 1, figsize=(6, 8))
+    def plot_interpretation(
+        self,
+        x: Dict[str, torch.Tensor],
+        output: Dict[str, torch.Tensor],
+        idx: int,
+        ax=None,
+        plot_seasonality_and_generic_on_secondary_axis: bool = False,
+    ) -> plt.Figure:
+        """
+        Plot interpretation.
+
+        Plot two pannels: prediction and backcast vs actuals and
+        decomposition of prediction into trend, seasonality and generic forecast.
+
+        Args:
+            x (Dict[str, torch.Tensor]): network input
+            output (Dict[str, torch.Tensor]): network output
+            idx (int): index of sample for which to plot the interpretation.
+            ax (List[matplotlib axes], optional): list of two matplotlib axes onto which to plot the interpretation.
+                Defaults to None.
+            plot_seasonality_and_generic_on_secondary_axis (bool, optional): if to plot seasonality and
+                generic forecast on secondary axis in second panel. Defaults to False.
+
+        Returns:
+            plt.Figure: matplotlib figure
+        """
+        if ax is None:
+            fig, ax = plt.subplots(2, 1, figsize=(6, 8))
+        else:
+            fig = ax.get_figure()
 
         time = torch.arange(-self.hparams.context_length, self.hparams.prediction_length)
 
+        def to_prediction(y):
+            return self.transform_output(dict(prediction=y[[idx]], target_scale=x["target_scale"][[idx]]))[0]
+
         # plot target vs prediction
-        ax[0].plot(time, torch.cat([x["encoder_target"], x["decoder_target"]]), label="target")
+        ax[0].plot(time, torch.cat([x["encoder_target"][idx], x["decoder_target"][idx]]), label="target")
         ax[0].plot(
             time,
             torch.cat(
                 [
-                    x["encoder_target"] - self.loss.to_prediction(output["backcast"].detach()),
-                    self.loss.to_prediction(output["prediction"].detach()),
+                    to_prediction(x["encoder_cont"][idx, :, 0] - output["backcast"].detach()),
+                    output["prediction"][idx].detach(),
                 ],
                 dim=0,
             ),
@@ -202,16 +266,18 @@ class NBeats(BaseModel):
         ax[0].set_xlabel("Time")
 
         # plot blocks
-        if self.loss.log_space:
+        if plot_seasonality_and_generic_on_secondary_axis:
             ax2 = ax[1].twinx()
             ax2.set_ylabel("Seasonality / Generic")
         else:
             ax2 = ax[1]
         for title in ["trend", "seasonality", "generic"]:
+            if title not in self.hparams.stack_types:
+                continue
             if title == "trend":
-                ax[1].plot(time, self.loss.to_prediction(output[title]), label=title.capitalize())
+                ax[1].plot(time, to_prediction(output[title]), label=title.capitalize())
             else:
-                ax2.plot(time, self.loss.to_prediction(output[title]), label=title.capitalize())
+                ax2.plot(time, to_prediction(output[title]), label=title.capitalize())
         ax[1].set_xlabel("Time")
         ax[1].set_ylabel("Trend")
 
